from twisted.python import components, log
from twisted.web import server, resource, soap
from twisted.internet import reactor, protocol, defer

try:
    from twisted.words.xish import domish, utility
except:
    from twisted.xish import domish, utility


import SOAPpy
import random 

import session


class SOAPPJab(soap.SOAPPublisher):
    """ An soap interface to punjab jabber connections
    """
    def __init__(self,v=0):
        soap.SOAPPublisher.__init__(self)
        self.counter  = 0
        self.sessions = {}
        self.v        = 0
    
    def soap_connect(self, jid, passwd, port=5222, rid = random.randint(10000,1000000)):
        """ Connect to a jabber server """
        attrs = {}
        try:
            attrs['user'],attrs['to'] = jid.split('@')
        except:
            attrs['user'] = jid
            attrs['to']   = jid
            
        if attrs['to'].find('/') != -1:
            attrs['to'], attrs['resource'] = attrs['to'].split('/')
        else:
            attrs['resource'] = "PunJab"
            
        attrs['rid'] = rid
        attrs['wait'] = 0
        attrs['hold'] = 0
        attrs['xml:lang'] = 'en' # this is english for now
        
        s, d = session.make_session(self,attrs)              
        
        d.addCallback(self.login, s, jid, passwd)
        d.addErrback(self.return_error)
        return d # this needs to be cleaned up, I need to understand this better

    def soap_is_connected(self, sid):
        if self.sessions.has_key(sid):
            return sid
        else:
            return 0

    def login(self, data, session, jid, passwd):
        # login to the jabber server        
        d = session.startLogin(jid,passwd)
        d.addCallback(self.return_login)
        d.addErrback(self.return_error)
        return d

    def return_login(self, session):
        # maybe do some other stuff?
        return session.sid

    def soap_disconnect(self,sid):
        """ disconnect from the jabber server and close your session. """
        try:
            if self.sessions.has_key(sid):
                s = self.sessions[sid]
                s.terminate()
            else:
                return SOAPpy.faultType( "Session Does not exists. Perhaps you are not logged in? ")
        except:
            return SOAPpy.faultType( "Error in disconnect.")
        
        return 1

    def soap_start_contact(self, sid, service, to, nick = '', group = '', request = 0):
        """ Subscribe to a jabber entities presence.
        """
        try:
            if self.sessions.has_key(sid):
                s = self.sessions[sid]        
                if request == 1:
                    s.sendPresence(to,'subscribe','Online','Online')

                d = s.setRoster(to,nick,group)
                d.addCallback(self.return_send)
                d.addErrback(self.return_error)
                return d
            else:
                return SOAPpy.faultType( "Session Does not exists. Perhaps you are not logged in? ")
        except:
            return SOAPpy.faultType( "Error in start contact")
        

    def soap_start_roster(self, sid):
        """ Request your jabber roster.
        """
        try:
            if self.sessions.has_key(sid):
                s = self.sessions[sid]        
            
                d = s.startRoster()
                d.addCallback(self.return_obj)
                d.addErrback(self.return_error)
                return d
            else:
                return SOAPpy.faultType( "Session Does not exists. Perhaps you are not logged in? ")
        except:
            raise # if debug then raise
            return SOAPpy.faultType( "Error in roster. ")

    soap_get_roster = soap_start_roster
    

    def soap_send_presence(self, sid, to = '', type = '', show = '', status = '', priority = 0):
        """ Send a presence packet to a jabber entity.
        """
        try:
            if self.sessions.has_key(sid):
                s = self.sessions[sid]        
                s.sendPresence(to,type,show,status,priority)
            else:
                return SOAPpy.faultType( "Session Does not exists. Perhaps you are not logged in? ")
        except:
            return SOAPpy.faultType( "Error in send presence. ")
        
        return 1

    def return_send(self, session):
        return 1
        
    def return_obj(self, obj):
        return obj

    def return_error(self, error):
        return SOAPpy.faultType("soap : " + str(error))
        
    error = return_error
    
    def soap_get_presence(self, sid, to = '', f = ''):
        """ Get a presence packet that has been sent to you.
        """
        try:
            if self.sessions.has_key(sid):
                s = self.sessions[sid]        
                
                return s.getPresence(to,f)
            else:
                return SOAPpy.faultType( "Session Does not exists. Perhaps you are not logged in? ")
        except:
            raise # if debug then raise
            return SOAPpy.faultType( "Error in get presence.")
        
        return 0

    def soap_send_message(self, sid, to, subject = '', body = '', type = 'chat', id = 0):
        """ Send a message packet to a jabber entity.
        """
        try:
            if self.sessions.has_key(sid):
                s = self.sessions[sid]        
                s.sendMessage(to,subject,body,type,id)
            else:
                return SOAPpy.faultType( "Session Does not exists. Perhaps you are not logged in? ")
        except:
            return SOAPpy.faultType( "Error in send message. ")
        
        return 1


    def soap_get_message(self, sid, to = '', type = '', id = 0, f = ''):
        """ get a message that has been sent to you and deletes it from punjab.
        """
        try:
            if self.sessions.has_key(sid):
                s = self.sessions[sid]        
            
                return s.getMessage(to,f, type, id)
            else:
                return SOAPpy.faultType( "Session Does not exists. Perhaps you are not logged in? ")
        except:
            raise # if debug then raise
            return SOAPpy.faultType( "Error in get_message.")
        
        return 0


    def soap_check_message(self, sid, to = '', type = '', id = 0, f = ''):
        """ get a message that has been sent to you and DO NOT delete it from punjab.
        """
        try:
            if self.sessions.has_key(sid):
                s = self.sessions[sid]        
                
                return s.checkMessage(to,f, type, id)
            else:
                return SOAPpy.faultType( "Session Does not exists. Perhaps you are not logged in? ")
        except:
            raise # if debug then raise
            return SOAPpy.faultType( "Error in check message. ")
        
        return 0

    def soap_get_iq(self, sid, id):
        """ get iq packets that have been sent to you.
        """
        try:
            if self.sessions.has_key(sid):
                s = self.sessions[sid]        
                
                return s.getIq(id)

            else:
                return SOAPpy.faultType( "Session Does not exists. Perhaps you are not logged in? ")
        except:
            raise # if debug then raise
            return SOAPpy.faultType( " Error in get_iq. ")
        
        return 0

    def soap_browse(self, sid, jid = '', auto = 0, items = 0):
        try:
            if self.sessions.has_key(sid):
                s = self.sessions[sid]                                
                                
                d = s.browse(jid,auto,items)                
                d.addCallback(self.return_obj)
                d.addErrback(self.return_error)
                return d 
                
            else:
                return SOAPpy.faultType( "Session Does not exists. Perhaps you are not logged in? ")
        except:
            raise
            return SOAPpy.faultType( "Error in browse. ")
        
        return 0

    def soap_vcard(self, sid, jid):
        try:
            if self.sessions.has_key(sid):
                s = self.sessions[sid]        
                
                d = s.vcard(jid)
                d.addCallback(self.return_obj)
                d.addErrback(self.return_error)
                return d 
            
            else:
                return SOAPpy.faultType( "Session Does not exists. Perhaps you are not logged in? ")
        except:
            raise # if debug then raise
            return SOAPpy.faultType( "Error in vcard ")
        
        return 0
        

    def soap_send_xml(self, sid):
        """ send any xml to the jabber server
        """
        log.msg(sid)
        return SOAPpy.faultType( "Not implemented.")
    
